"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.click = click;
exports.dblClick = dblClick;
exports.tripleClick = tripleClick;

var _utils = require("./utils");

function click(element, {
  skipHover = false,
  skipPointerEventsCheck = false
} = {}) {
  if (!skipPointerEventsCheck && !(0, _utils.hasPointerEvents)(element)) {
    throw new Error('unable to click element as it has or inherits pointer-events set to "none".');
  } // istanbul ignore else


  if (!skipHover) // We just checked for `pointerEvents`. We can always skip this one in `hover`.
    this.hover(element, {
      skipPointerEventsCheck: true
    });
  this.pointer({
    keys: '[MouseLeft]',
    target: element
  });
}

function dblClick(element, {
  skipPointerEventsCheck = false
} = {}) {
  if (!skipPointerEventsCheck && !(0, _utils.hasPointerEvents)(element)) {
    throw new Error('unable to double-click element as it has or inherits pointer-events set to "none".');
  }

  this.hover(element, {
    skipPointerEventsCheck: true
  });
  this.pointer({
    keys: '[MouseLeft][MouseLeft]',
    target: element
  });
}

function tripleClick(element, {
  skipPointerEventsCheck = false
} = {}) {
  if (!skipPointerEventsCheck && !(0, _utils.hasPointerEvents)(element)) {
    throw new Error('unable to triple-click element as it has or inherits pointer-events set to "none".');
  }

  this.hover(element, {
    skipPointerEventsCheck: true
  });
  this.pointer({
    keys: '[MouseLeft][MouseLeft][MouseLeft]',
    target: element
  });
}