"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prepareInterceptor = prepareInterceptor;
const Interceptor = Symbol('Interceptor for programmatical calls');

function prepareInterceptor(element, propName, interceptorImpl) {
  const prototypeDescriptor = Object.getOwnPropertyDescriptor(element.constructor.prototype, propName);
  const objectDescriptor = Object.getOwnPropertyDescriptor(element, propName);
  const target = prototypeDescriptor != null && prototypeDescriptor.set ? 'set' : 'value';

  if (typeof (prototypeDescriptor == null ? void 0 : prototypeDescriptor[target]) !== 'function' || prototypeDescriptor[target][Interceptor]) {
    return;
  }

  function intercept(...args) {
    const {
      applyNative = true,
      realArgs
    } = interceptorImpl.call(this, ...args);
    const realFunc = (!applyNative && objectDescriptor || prototypeDescriptor)[target];

    if (target === 'set') {
      realFunc.call(this, realArgs);
    } else {
      realFunc.call(this, ...realArgs);
    }
  }

  ;
  intercept[Interceptor] = Interceptor;
  Object.defineProperty(element, propName, { ...(objectDescriptor != null ? objectDescriptor : prototypeDescriptor),
    [target]: intercept
  });
}