"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearUISelection = clearUISelection;
exports.getUISelection = getUISelection;
exports.prepareSelectionInterceptor = prepareSelectionInterceptor;
exports.setUISelection = setUISelection;

var _ = require(".");

var _interceptor = require("./interceptor");

const UISelection = Symbol('Displayed selection in UI');

function prepareSelectionInterceptor(element) {
  (0, _interceptor.prepareInterceptor)(element, 'setSelectionRange', function interceptorImpl(start, end, direction = 'none') {
    const isUI = start && typeof start === 'object' && start[UISelection];

    if (!isUI) {
      this[UISelection] = undefined;
    }

    return {
      realArgs: [Number(start), end, direction]
    };
  });
  (0, _interceptor.prepareInterceptor)(element, 'selectionStart', function interceptorImpl(v) {
    this[UISelection] = undefined;
    return {
      realArgs: v
    };
  });
  (0, _interceptor.prepareInterceptor)(element, 'selectionEnd', function interceptorImpl(v) {
    this[UISelection] = undefined;
    return {
      realArgs: v
    };
  });
}

function setUISelection(element, {
  focusOffset: focusOffsetParam,
  anchorOffset: anchorOffsetParam = focusOffsetParam
}, mode = 'replace') {
  const valueLength = (0, _.getUIValue)(element).length;

  const sanitizeOffset = o => Math.max(0, Math.min(valueLength, o));

  const anchorOffset = mode === 'replace' || element[UISelection] === undefined ? sanitizeOffset(anchorOffsetParam) : element[UISelection].anchorOffset;
  const focusOffset = sanitizeOffset(focusOffsetParam);
  const startOffset = Math.min(anchorOffset, focusOffset);
  const endOffset = Math.max(anchorOffset, focusOffset);
  element[UISelection] = {
    anchorOffset,
    focusOffset
  };

  if (element.selectionStart === startOffset && element.selectionEnd === endOffset) {
    return;
  } // eslint-disable-next-line no-new-wrappers


  const startObj = new Number(startOffset);
  startObj[UISelection] = UISelection;

  try {
    element.setSelectionRange(startObj, endOffset);
  } catch {// DOMException for invalid state is expected when calling this
    // on an element without support for setSelectionRange
  }
}

function getUISelection(element) {
  var _element$UISelection, _element$selectionSta, _element$selectionEnd;

  const sel = (_element$UISelection = element[UISelection]) != null ? _element$UISelection : {
    anchorOffset: (_element$selectionSta = element.selectionStart) != null ? _element$selectionSta : 0,
    focusOffset: (_element$selectionEnd = element.selectionEnd) != null ? _element$selectionEnd : 0
  };
  return { ...sel,
    startOffset: Math.min(sel.anchorOffset, sel.focusOffset),
    endOffset: Math.max(sel.anchorOffset, sel.focusOffset)
  };
}

function clearUISelection(element) {
  element[UISelection] = undefined;
}