"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.endTrackValue = endTrackValue;
exports.getInitialValue = getInitialValue;
exports.getUIValue = getUIValue;
exports.prepareValueInterceptor = prepareValueInterceptor;
exports.setInitialValue = setInitialValue;
exports.setUIValue = setUIValue;
exports.startTrackValue = startTrackValue;

var _interceptor = require("./interceptor");

var _selection = require("./selection");

const UIValue = Symbol('Displayed value in UI');
const InitialValue = Symbol('Initial value to compare on blur');
const TrackChanges = Symbol('Track programmatic changes for React workaround');

function valueInterceptor(v) {
  const isUI = typeof v === 'object' && v[UIValue];
  this[UIValue] = isUI ? String(v) : undefined;

  if (!isUI) {
    trackValue(this, String(v));
    this[InitialValue] = String(v); // Programmatically setting the value property
    // moves the cursor to the end of the input.

    (0, _selection.clearUISelection)(this);
  }

  return {
    applyNative: !!isUI,
    realArgs: String(v)
  };
}

function prepareValueInterceptor(element) {
  (0, _interceptor.prepareInterceptor)(element, 'value', valueInterceptor);
}

function setUIValue(element, value) {
  element.value = {
    [UIValue]: UIValue,
    toString: () => value
  };
}

function getUIValue(element) {
  return element[UIValue] === undefined ? element.value : String(element[UIValue]);
}

function setInitialValue(element) {
  element[InitialValue] = element.value;
}

function getInitialValue(element) {
  return element[InitialValue];
}

function startTrackValue(element) {
  element[TrackChanges] = [];
}

function trackValue(element, v) {
  var _element$TrackChanges;

  (_element$TrackChanges = element[TrackChanges]) == null ? void 0 : _element$TrackChanges.push(v);
}

function endTrackValue(element) {
  const tracked = element[TrackChanges];
  element[TrackChanges] = undefined;
  return tracked;
}