"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createPointerState = createPointerState;
exports.pointer = pointer;
exports.pointerImplementationWrapper = pointerImplementationWrapper;

var _dom = require("@testing-library/dom");

var _keyboard = require("../keyboard");

var _parseKeyDef = require("./parseKeyDef");

var _keyMap = require("./keyMap");

var _pointerAction = require("./pointerAction");

function pointer(input, options = {}) {
  var _options$delay;

  const {
    promise,
    pointerState
  } = pointerImplementationWrapper(input, options);

  if (((_options$delay = options.delay) != null ? _options$delay : 0) > 0) {
    return (0, _dom.getConfig)().asyncWrapper(() => promise.then(() => pointerState));
  } else {
    // prevent users from dealing with UnhandledPromiseRejectionWarning in sync call
    promise.catch(console.error);
    return pointerState;
  }
}

function pointerImplementationWrapper(input, config) {
  const {
    pointerState = createPointerState(),
    keyboardState = (0, _keyboard.createKeyboardState)(),
    delay = 0,
    pointerMap = _keyMap.defaultKeyMap
  } = config;
  const options = {
    delay,
    pointerMap
  };
  const actions = [];
  (Array.isArray(input) ? input : [input]).forEach(actionInput => {
    if (typeof actionInput === 'string') {
      actions.push(...(0, _parseKeyDef.parseKeyDef)(actionInput, options));
    } else if ('keys' in actionInput) {
      actions.push(...(0, _parseKeyDef.parseKeyDef)(actionInput.keys, options).map(i => ({ ...actionInput,
        ...i
      })));
    } else {
      actions.push(actionInput);
    }
  });
  return {
    promise: (0, _pointerAction.pointerAction)(actions, options, {
      pointerState,
      keyboardState
    }),
    pointerState
  };
}

function createPointerState() {
  return {
    pointerId: 1,
    position: {
      mouse: {
        pointerType: 'mouse',
        pointerId: 1,
        coords: {
          clientX: 0,
          clientY: 0,
          offsetX: 0,
          offsetY: 0,
          pageX: 0,
          pageY: 0,
          x: 0,
          y: 0
        }
      }
    },
    pressed: []
  };
}