"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pointerAction = pointerAction;

var _utils = require("../utils");

var _pointerMove = require("./pointerMove");

var _pointerPress = require("./pointerPress");

async function pointerAction(actions, options, state) {
  const ret = [];

  for (let i = 0; i < actions.length; i++) {
    var _action$target, _action$coords;

    const action = actions[i];
    const pointerName = 'pointerName' in action && action.pointerName ? action.pointerName : 'keyDef' in action ? action.keyDef.pointerType === 'touch' ? action.keyDef.name : action.keyDef.pointerType : 'mouse';
    const target = (_action$target = action.target) != null ? _action$target : getPrevTarget(pointerName, state.pointerState);
    const coords = (_action$coords = action.coords) != null ? _action$coords : pointerName in state.pointerState.position ? state.pointerState.position[pointerName].coords : undefined;
    const promise = 'keyDef' in action ? (0, _pointerPress.pointerPress)({ ...action,
      target,
      coords
    }, state) : (0, _pointerMove.pointerMove)({ ...action,
      target,
      coords
    }, state);
    ret.push(promise);

    if (options.delay > 0) {
      await promise;

      if (i < actions.length - 1) {
        await (0, _utils.wait)(options.delay);
      }
    }
  }

  delete state.pointerState.activeClickCount;
  return Promise.all(ret);
}

function getPrevTarget(pointerName, state) {
  if (!(pointerName in state.position) || !state.position[pointerName].target) {
    throw new Error('This pointer has no previous position. Provide a target property!');
  }

  return state.position[pointerName].target;
}