"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pointerPress = pointerPress;

var _dom = require("@testing-library/dom");

var _utils = require("../utils");

var _document = require("../document");

var _resolveSelectionTarget = require("./resolveSelectionTarget");

/* eslint-disable complexity */
async function pointerPress(action, state) {
  const {
    keyDef,
    target,
    releasePrevious,
    releaseSelf
  } = action;
  const previous = state.pointerState.pressed.find(p => p.keyDef === keyDef);
  const pointerName = keyDef.pointerType === 'touch' ? keyDef.name : keyDef.pointerType;
  const targetIsDisabled = (0, _utils.isDisabled)(target);

  if (previous) {
    up(state, pointerName, action, previous, targetIsDisabled);
  }

  if (!releasePrevious) {
    const press = down(state, pointerName, action, targetIsDisabled);

    if (releaseSelf) {
      up(state, pointerName, action, press, targetIsDisabled);
    }
  }
}

function getNextPointerId(state) {
  state.pointerId = state.pointerId + 1;
  return state.pointerId;
}

function down({
  pointerState,
  keyboardState
}, pointerName, {
  keyDef,
  node,
  offset,
  target,
  coords
}, targetIsDisabled) {
  var _pointerState$activeC, _pointerState$activeC2, _pointerState$activeC3;

  const {
    name,
    pointerType,
    button
  } = keyDef;
  const pointerId = pointerType === 'mouse' ? 1 : getNextPointerId(pointerState);
  pointerState.position[pointerName] = { ...pointerState.position[pointerName],
    pointerId,
    pointerType,
    target,
    coords
  };
  let isMultiTouch = false;
  let isPrimary = true;

  if (pointerType !== 'mouse') {
    for (const obj of pointerState.pressed) {
      // TODO: test multi device input across browsers
      // istanbul ignore else
      if (obj.keyDef.pointerType === pointerType) {
        obj.isMultiTouch = true;
        isMultiTouch = true;
        isPrimary = false;
      }
    }
  }

  if (((_pointerState$activeC = pointerState.activeClickCount) == null ? void 0 : _pointerState$activeC[0]) !== name) {
    delete pointerState.activeClickCount;
  }

  const clickCount = Number((_pointerState$activeC2 = (_pointerState$activeC3 = pointerState.activeClickCount) == null ? void 0 : _pointerState$activeC3[1]) != null ? _pointerState$activeC2 : 0) + 1;
  pointerState.activeClickCount = [name, clickCount];
  const pressObj = {
    keyDef,
    downTarget: target,
    pointerId,
    unpreventedDefault: true,
    isMultiTouch,
    isPrimary,
    clickCount
  };
  pointerState.pressed.push(pressObj);

  if (!targetIsDisabled) {
    if (pointerType !== 'mouse') {
      fire('pointerover');
      fire('pointerenter');
    }

    if (pointerType !== 'mouse' || !pointerState.pressed.some(p => p.keyDef !== keyDef && p.keyDef.pointerType === pointerType)) {
      fire('pointerdown');
    }

    if (pointerType === 'mouse') {
      pressObj.unpreventedDefault = fire('mousedown');
    } // TODO: touch...

  }

  if (pointerType === 'mouse' && pressObj.unpreventedDefault) {
    mousedownDefaultBehavior({
      target,
      targetIsDisabled,
      clickCount,
      position: pointerState.position[pointerName],
      node,
      offset
    });
  }

  return pressObj;

  function fire(type) {
    return (0, _utils.firePointerEvent)(target, type, {
      pointerState,
      keyboardState,
      button,
      clickCount,
      coords,
      isPrimary,
      pointerId,
      pointerType
    });
  }
}

function up({
  pointerState,
  keyboardState
}, pointerName, {
  keyDef: {
    pointerType,
    button
  },
  target,
  coords,
  node,
  offset
}, pressed, targetIsDisabled) {
  pointerState.pressed = pointerState.pressed.filter(p => p !== pressed);
  const {
    isMultiTouch,
    isPrimary,
    pointerId,
    clickCount
  } = pressed;
  let {
    unpreventedDefault
  } = pressed;
  pointerState.position[pointerName] = { ...pointerState.position[pointerName],
    target,
    coords
  }; // TODO: pointerleave for touch device

  if (!targetIsDisabled) {
    if (pointerType !== 'mouse' || !pointerState.pressed.filter(p => p.keyDef.pointerType === pointerType).length) {
      fire('pointerup');
    }

    if (pointerType !== 'mouse') {
      fire('pointerout');
      fire('pointerleave');
    }

    if (pointerType !== 'mouse' && !isMultiTouch) {
      if (clickCount === 1) {
        fire('mouseover');
        fire('mouseenter');
      }

      fire('mousemove');
      unpreventedDefault = fire('mousedown') && unpreventedDefault;
    }
  }

  if (unpreventedDefault && pointerType !== 'mouse' && !isMultiTouch) {
    mousedownDefaultBehavior({
      target,
      targetIsDisabled,
      clickCount,
      position: pointerState.position[pointerName],
      node,
      offset
    });
  }

  delete pointerState.position[pointerName].selectionRange;

  if (!targetIsDisabled) {
    if (pointerType === 'mouse' || !isMultiTouch) {
      unpreventedDefault = fire('mouseup') && unpreventedDefault;
      const canClick = pointerType !== 'mouse' || button === 'primary';

      if (canClick && target === pressed.downTarget) {
        fire('click');

        if (clickCount === 2) {
          fire('dblclick');
        } // If the click happens inside a `label` with a control, the control (or closes focusable) is focused.


        const label = target.closest('label');

        if (label != null && label.control) {
          var _findClosest;

          (0, _utils.focus)((_findClosest = (0, _utils.findClosest)(label.control, _utils.isFocusable)) != null ? _findClosest : target.ownerDocument.body);
        }
      }

      if (pointerType === 'mouse' && button === 'secondary') {
        _dom.fireEvent.contextMenu(target);
      }
    }
  }

  function fire(type) {
    return (0, _utils.firePointerEvent)(target, type, {
      pointerState,
      keyboardState,
      button,
      clickCount,
      coords,
      isPrimary,
      pointerId,
      pointerType
    });
  }
}

function mousedownDefaultBehavior({
  position,
  target,
  targetIsDisabled,
  clickCount,
  node,
  offset
}) {
  var _findClosest2;

  // The closest focusable element is focused when a `mousedown` would have been fired.
  // Even if there was no `mousedown` because the element was disabled.
  // A `mousedown` that preventsDefault cancels this though.
  (0, _utils.focus)((_findClosest2 = (0, _utils.findClosest)(target, _utils.isFocusable)) != null ? _findClosest2 : target.ownerDocument.body); // TODO: What happens if a focus event handler interfers?
  // An unprevented mousedown moves the cursor to the closest character.
  // We try to approximate the behavior for a no-layout environment.

  if (!targetIsDisabled) {
    const hasValue = (0, _utils.isElementType)(target, ['input', 'textarea']); // On non-input elements the text selection per multiple click
    // can extend beyond the target boundaries.
    // The exact mechanism what is considered in the same line is unclear.
    // Looks it might be every inline element.
    // TODO: Check what might be considered part of the same line of text.

    const text = String(hasValue ? (0, _document.getUIValue)(target) : target.textContent);
    const [start, end] = node ? // As offset is describing a DOMOffset it is non-trivial to determine
    // which elements might be considered in the same line of text.
    // TODO: support expanding initial range on multiple clicks if node is given
    [offset, offset] : getTextRange(text, offset, clickCount); // TODO: implement modifying selection per shift/ctrl+mouse

    if (hasValue) {
      (0, _document.setUISelection)(target, {
        anchorOffset: start != null ? start : text.length,
        focusOffset: end != null ? end : text.length
      });
      position.selectionRange = {
        node: target,
        start: start != null ? start : 0,
        end: end != null ? end : text.length
      };
    } else {
      const {
        node: startNode,
        offset: startOffset
      } = (0, _resolveSelectionTarget.resolveSelectionTarget)({
        target,
        node,
        offset: start
      });
      const {
        node: endNode,
        offset: endOffset
      } = (0, _resolveSelectionTarget.resolveSelectionTarget)({
        target,
        node,
        offset: end
      });
      const range = new Range();
      range.setStart(startNode, startOffset);
      range.setEnd(endNode, endOffset);
      position.selectionRange = range;
      const selection = target.ownerDocument.getSelection();
      selection.removeAllRanges();
      selection.addRange(range.cloneRange());
    }
  }
}

function getTextRange(text, pos, clickCount) {
  if (clickCount % 3 === 1 || text.length === 0) {
    return [pos, pos];
  }

  const textPos = pos != null ? pos : text.length;

  if (clickCount % 3 === 2) {
    return [textPos - text.substr(0, pos).match(/(\w+|\s+|\W)?$/)[0].length, pos === undefined ? pos : pos + text.substr(pos).match(/^(\w+|\s+|\W)?/)[0].length];
  } // triple click


  return [textPos - text.substr(0, pos).match(/[^\r\n]*$/)[0].length, pos === undefined ? pos : pos + text.substr(pos).match(/^[^\r\n]*/)[0].length];
}