import { clear } from './clear';
import { click, clickOptions, dblClick, tripleClick } from './click';
import { hover, unhover } from './hover';
import { keyboard, keyboardOptions } from './keyboard';
import type { keyboardState } from './keyboard/types';
import { paste } from './paste';
import { pointer } from './pointer';
import type { pointerOptions, pointerState } from './pointer/types';
import { deselectOptions, selectOptions } from './selectOptions';
import { tab, tabOptions } from './tab';
import { type, typeOptions } from './type';
import { upload, uploadOptions } from './upload';
import { PointerOptions } from './utils';
export declare const userEventApis: {
    clear: typeof clear;
    click: typeof click;
    dblClick: typeof dblClick;
    deselectOptions: typeof deselectOptions;
    hover: typeof hover;
    keyboard: typeof keyboard;
    paste: typeof paste;
    pointer: typeof pointer;
    selectOptions: typeof selectOptions;
    tab: typeof tab;
    tripleClick: typeof tripleClick;
    type: typeof type;
    unhover: typeof unhover;
    upload: typeof upload;
};
export declare type UserEventApis = typeof userEventApis;
declare type setup = ReturnType<typeof _setup>['setup'];
export declare type UserEvent = UserEventApis & {
    setup: setup;
};
declare type ClickOptions = Omit<clickOptions, 'clickCount'>;
declare type KeyboardOptions = Partial<keyboardOptions>;
declare type PointerApiOptions = Partial<pointerOptions>;
declare type TabOptions = Omit<tabOptions, 'shift'>;
declare type TypeOptions = Omit<typeOptions, 'initialSelectionStart' | 'initialSelectionEnd'>;
declare type UploadOptions = uploadOptions;
interface SetupOptions extends ClickOptions, KeyboardOptions, PointerOptions, PointerApiOptions, TabOptions, TypeOptions, UploadOptions {
}
/**
 * Start a "session" with userEvent.
 * All APIs returned by this function share an input device state and a default configuration.
 */
export declare function setup(options?: SetupOptions): {
    clear: typeof clear;
    click: typeof click;
    dblClick: typeof dblClick;
    deselectOptions: typeof deselectOptions;
    hover: typeof hover;
    keyboard: typeof keyboard;
    paste: typeof paste;
    pointer: typeof pointer;
    selectOptions: typeof selectOptions;
    tab: typeof tab;
    tripleClick: typeof tripleClick;
    type: typeof type;
    unhover: typeof unhover;
    upload: typeof upload;
} & {
    /**
     * Create a set of callbacks with different default settings but the same state.
     */
    setup(options: SetupOptions): {
        clear: typeof clear;
        click: typeof click;
        dblClick: typeof dblClick;
        deselectOptions: typeof deselectOptions;
        hover: typeof hover;
        keyboard: typeof keyboard;
        paste: typeof paste;
        pointer: typeof pointer;
        selectOptions: typeof selectOptions;
        tab: typeof tab;
        tripleClick: typeof tripleClick;
        type: typeof type;
        unhover: typeof unhover;
        upload: typeof upload;
    } & any;
};
declare function _setup({ applyAccept, autoModify, delay, document, keyboardMap, pointerMap, skipAutoClose, skipClick, skipHover, skipPointerEventsCheck, }: SetupOptions, { keyboardState, pointerState, }: {
    keyboardState: keyboardState;
    pointerState: pointerState;
}): UserEventApis & {
    /**
     * Create a set of callbacks with different default settings but the same state.
     */
    setup(options: SetupOptions): ReturnType<typeof _setup>;
};
export {};
