"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.type = type;

var _dom = require("@testing-library/dom");

var _document = require("./document");

var _utils = require("./utils");

var _keyboard = require("./keyboard");

// this needs to be wrapped in the event/asyncWrapper for React's act and angular's change detection
// depending on whether it will be async.
function type(element, text, {
  delay = 0,
  ...options
} = {}) {
  (0, _document.prepareDocument)(element.ownerDocument); // we do not want to wrap in the asyncWrapper if we're not
  // going to actually be doing anything async, so we only wrap
  // if the delay is greater than 0

  if (delay > 0) {
    return (0, _dom.getConfig)().asyncWrapper(() => typeImplementation(this, element, text, {
      delay,
      ...options
    }));
  } else {
    return void typeImplementation(this, element, text, {
      delay,
      ...options
    }) // prevents users from dealing with UnhandledPromiseRejectionWarning
    .catch(console.error);
  }
}

async function typeImplementation(userEvent, element, text, {
  delay,
  skipClick = false,
  skipAutoClose = false,
  initialSelectionStart = undefined,
  initialSelectionEnd = undefined
}) {
  // TODO: properly type guard
  // we use this workaround for now to prevent changing behavior
  if (element.disabled) return;
  if (!skipClick) userEvent.click(element);

  if (initialSelectionStart !== undefined) {
    (0, _utils.setSelectionRange)(element, initialSelectionStart, initialSelectionEnd != null ? initialSelectionEnd : initialSelectionStart);
  }

  const {
    promise,
    releaseAllKeys
  } = (0, _keyboard.keyboardImplementationWrapper)(text, {
    delay,
    document: element.ownerDocument
  });

  if (delay > 0) {
    await promise;
  }

  if (!skipAutoClose) {
    releaseAllKeys();
  } // eslint-disable-next-line consistent-return -- we need to return the internal Promise so that it is catchable if we don't await


  return promise;
}