"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateNewValue = calculateNewValue;

var _document = require("../../document");

var _isValidDateValue = require("./isValidDateValue");

var _isValidInputTimeValue = require("./isValidInputTimeValue");

/**
 * Calculate a new text value.
 */
function calculateNewValue(inputData, node, {
  startOffset,
  endOffset
}, inputType) {
  const value = (0, _document.getUIValue)(node);
  const prologEnd = Math.max(0, startOffset === endOffset && inputType === 'deleteContentBackward' ? startOffset - 1 : startOffset);
  const prolog = value.substring(0, prologEnd);
  const epilogStart = Math.min(value.length, startOffset === endOffset && inputType === 'deleteContentForward' ? startOffset + 1 : endOffset);
  const epilog = value.substring(epilogStart, value.length);
  let newValue = `${prolog}${inputData}${epilog}`;
  const newOffset = prologEnd + inputData.length;

  if (node.type === 'date' && !(0, _isValidDateValue.isValidDateValue)(node, newValue)) {
    newValue = value;
  }

  if (node.type === 'time' && !(0, _isValidInputTimeValue.isValidInputTimeValue)(node, newValue)) {
    if ((0, _isValidInputTimeValue.isValidInputTimeValue)(node, inputData)) {
      newValue = inputData;
    } else {
      newValue = value;
    }
  }

  return {
    oldValue: value,
    newValue,
    newOffset
  };
}