"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.editInputElement = editInputElement;

var _dom = require("@testing-library/dom");

var _document = require("../../document");

var _selection = require("../focus/selection");

/**
 * Change the value of an element as if it was changed as a result of a user input.
 *
 * Fires the input event.
 */
function editInputElement(element, {
  newValue,
  newSelection,
  eventOverrides
}) {
  const oldValue = element.value; // apply the changes before firing the input event, so that input handlers can access the altered dom and selection

  (0, _document.setUIValue)(element, newValue);
  (0, _selection.setSelection)({
    focusNode: newSelection.node,
    anchorOffset: newSelection.offset,
    focusOffset: newSelection.offset
  }); // When the input event happens in the browser, React executes all event handlers
  // and if they change state of a controlled value, nothing happens.
  // But when we trigger the event handlers in test environment,
  // the changes are rolled back by React before the state update is applied.
  // Then the updated state is applied which results in a resetted cursor.
  // There is probably a better way to work around  if we figure out
  // why the batched update is executed differently in our test environment.

  (0, _document.startTrackValue)(element);

  _dom.fireEvent.input(element, { ...eventOverrides
  });

  const tracked = (0, _document.endTrackValue)(element);

  if ((tracked == null ? void 0 : tracked.length) === 2 && tracked[0] === oldValue && tracked[1] === newValue) {
    (0, _selection.setSelection)({
      focusNode: newSelection.node,
      anchorOffset: newSelection.offset,
      focusOffset: newSelection.offset
    });
  }
}