"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prepareInput = prepareInput;

var _dom = require("@testing-library/dom");

var _utils = require("../../utils");

function prepareInput(data, element, inputType = 'insertText') {
  const inputRange = (0, _utils.getInputRange)(element);
  /* istanbul ignore if */

  if (!inputRange) {
    return;
  }

  if ('startContainer' in inputRange) {
    return {
      commit: () => {
        const del = !inputRange.collapsed;

        if (del) {
          inputRange.deleteContents();
        }

        if (data) {
          if (inputRange.endContainer.nodeType === 3) {
            const offset = inputRange.endOffset;
            inputRange.endContainer.insertData(offset, data);
            inputRange.setStart(inputRange.endContainer, offset + data.length);
            inputRange.setEnd(inputRange.endContainer, offset + data.length);
          } else {
            const text = element.ownerDocument.createTextNode(data);
            inputRange.insertNode(text);
            inputRange.setStart(text, data.length);
            inputRange.setEnd(text, data.length);
          }
        }

        if (del || data) {
          _dom.fireEvent.input(element, {
            inputType
          });
        }
      }
    };
  } else {
    return {
      getNewValue: () => (0, _utils.calculateNewValue)(data, element, inputRange, inputType).newValue,
      commit: () => {
        const {
          newValue,
          newOffset,
          oldValue
        } = (0, _utils.calculateNewValue)(data, element, inputRange, inputType);

        if (newValue === oldValue && newOffset === inputRange.startOffset && newOffset === inputRange.endOffset) {
          return;
        }

        (0, _utils.editInputElement)(element, {
          newValue,
          newSelection: {
            node: element,
            offset: newOffset
          },
          eventOverrides: {
            inputType
          }
        });
      }
    };
  }
}