"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getTabDestination = getTabDestination;

var _isDisabled = require("../misc/isDisabled");

var _isElementType = require("../misc/isElementType");

var _isVisible = require("../misc/isVisible");

var _selector = require("./selector");

function getTabDestination(activeElement, shift) {
  const document = activeElement.ownerDocument;
  const focusableElements = document.querySelectorAll(_selector.FOCUSABLE_SELECTOR);
  const enabledElements = Array.from(focusableElements).filter(el => el === activeElement || el.getAttribute('tabindex') !== '-1' && !(0, _isDisabled.isDisabled)(el) && // Hidden elements are not tabable
  (0, _isVisible.isVisible)(el));

  if (activeElement.getAttribute('tabindex') !== '-1') {
    // tabindex has no effect if the active element has tabindex="-1"
    enabledElements.sort((a, b) => Number(a.getAttribute('tabindex')) - Number(b.getAttribute('tabindex')));
  }

  const checkedRadio = {};
  let prunedElements = [document.body];
  const activeRadioGroup = (0, _isElementType.isElementType)(activeElement, 'input', {
    type: 'radio'
  }) ? activeElement.name : undefined;
  enabledElements.forEach(currentElement => {
    const el = currentElement; // For radio groups keep only the active radio
    // If there is no active radio, keep only the checked radio
    // If there is no checked radio, treat like everything else

    if ((0, _isElementType.isElementType)(el, 'input', {
      type: 'radio'
    }) && el.name) {
      // If the active element is part of the group, add only that
      if (el === activeElement) {
        prunedElements.push(el);
        return;
      } else if (el.name === activeRadioGroup) {
        return;
      } // If we stumble upon a checked radio, remove the others


      if (el.checked) {
        prunedElements = prunedElements.filter(e => !(0, _isElementType.isElementType)(e, 'input', {
          type: 'radio',
          name: el.name
        }));
        prunedElements.push(el);
        checkedRadio[el.name] = el;
        return;
      } // If we already found the checked one, skip


      if (typeof checkedRadio[el.name] !== 'undefined') {
        return;
      }
    }

    prunedElements.push(el);
  });
  const currentIndex = prunedElements.findIndex(el => el === activeElement);
  const nextIndex = shift ? currentIndex - 1 : currentIndex + 1;
  const defaultIndex = shift ? prunedElements.length - 1 : 0;
  return prunedElements[nextIndex] || prunedElements[defaultIndex];
}