"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isAllSelected = isAllSelected;
exports.selectAll = selectAll;

var _document = require("../../document");

var _isContentEditable = require("../edit/isContentEditable");

var _isEditable = require("../edit/isEditable");

var _isElementType = require("../misc/isElementType");

var _selection = require("./selection");

/**
 * Expand a selection like the browser does when pressing Ctrl+A.
 */
function selectAll(target) {
  var _getContentEditable;

  if ((0, _isElementType.isElementType)(target, 'textarea') || (0, _isElementType.isElementType)(target, 'input') && target.type in _isEditable.editableInputTypes) {
    return (0, _selection.setSelection)({
      focusNode: target,
      anchorOffset: 0,
      focusOffset: (0, _document.getUIValue)(target).length
    });
  }

  const focusNode = (_getContentEditable = (0, _isContentEditable.getContentEditable)(target)) != null ? _getContentEditable : target.ownerDocument.body;
  (0, _selection.setSelection)({
    focusNode,
    anchorOffset: 0,
    focusOffset: focusNode.childNodes.length
  });
}

function isAllSelected(target) {
  var _getContentEditable2;

  if ((0, _isElementType.isElementType)(target, 'textarea') || (0, _isElementType.isElementType)(target, 'input') && target.type in _isEditable.editableInputTypes) {
    return (0, _document.getUISelection)(target).startOffset === 0 && (0, _document.getUISelection)(target).endOffset === (0, _document.getUIValue)(target).length;
  }

  const focusNode = (_getContentEditable2 = (0, _isContentEditable.getContentEditable)(target)) != null ? _getContentEditable2 : target.ownerDocument.body;
  const selection = target.ownerDocument.getSelection();
  return (selection == null ? void 0 : selection.anchorNode) === focusNode && selection.focusNode === focusNode && selection.anchorOffset === 0 && selection.focusOffset === focusNode.childNodes.length;
}