"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.firePointerEvent = firePointerEvent;

var _dom = require("@testing-library/dom");

var _eventMap = require("@testing-library/dom/dist/event-map");

var _mouseButtons = require("./mouseButtons");

function firePointerEvent(target, type, {
  pointerState,
  keyboardState,
  pointerType,
  button,
  coords,
  pointerId,
  isPrimary,
  clickCount
}) {
  const init = { ...coords,
    altKey: keyboardState.modifiers.alt,
    ctrlKey: keyboardState.modifiers.ctrl,
    metaKey: keyboardState.modifiers.meta,
    shiftKey: keyboardState.modifiers.shift
  };

  if (type === 'click' || type.startsWith('pointer')) {
    init.pointerId = pointerId;
    init.pointerType = pointerType;
  }

  if (['pointerdown', 'pointerup'].includes(type)) {
    init.isPrimary = isPrimary;
  }

  if (['mousedown', 'mouseup', 'pointerdown', 'pointerup', 'click'].includes(type)) {
    init.button = (0, _mouseButtons.getMouseButton)(button != null ? button : 0);
    init.buttons = (0, _mouseButtons.getMouseButtons)(...pointerState.pressed.filter(p => p.keyDef.pointerType === pointerType).map(p => {
      var _p$keyDef$button;

      return (_p$keyDef$button = p.keyDef.button) != null ? _p$keyDef$button : 0;
    }));
  }

  if (['mousedown', 'mouseup', 'click', 'dblclick'].includes(type)) {
    init.detail = clickCount;
  }

  const eventKey = Object.keys(_eventMap.eventMap).find(k => k.toLowerCase() === type);

  const event = _dom.createEvent[eventKey](target, init); // see https://github.com/testing-library/react-testing-library/issues/268


  assignPositionInit(event, init);
  assignPointerInit(event, init);
  return (0, _dom.fireEvent)(target, event);
}

function assignProps(obj, props) {
  for (const [key, value] of Object.entries(props)) {
    Object.defineProperty(obj, key, {
      get: () => value
    });
  }
}

function assignPositionInit(obj, {
  x,
  y,
  clientX,
  clientY,
  offsetX,
  offsetY,
  pageX,
  pageY,
  screenX,
  screenY
}) {
  var _ref, _ref2, _ref3, _ref4;

  assignProps(obj, {
    /* istanbul ignore start */
    x: (_ref = x != null ? x : clientX) != null ? _ref : 0,
    y: (_ref2 = y != null ? y : clientY) != null ? _ref2 : 0,
    clientX: (_ref3 = x != null ? x : clientX) != null ? _ref3 : 0,
    clientY: (_ref4 = y != null ? y : clientY) != null ? _ref4 : 0,
    offsetX: offsetX != null ? offsetX : 0,
    offsetY: offsetY != null ? offsetY : 0,
    pageX: pageX != null ? pageX : 0,
    pageY: pageY != null ? pageY : 0,
    screenX: screenX != null ? screenX : 0,
    screenY: screenY != null ? screenY : 0
    /* istanbul ignore end */

  });
}

function assignPointerInit(obj, {
  isPrimary,
  pointerId,
  pointerType
}) {
  assignProps(obj, {
    isPrimary,
    pointerId,
    pointerType
  });
}